module Capybara::Poltergeist
  module NetworkTraffic
    require 'capybara/poltergeist/network_traffic/request'
    require 'capybara/poltergeist/network_traffic/response'
    require 'capybara/poltergeist/network_traffic/error'
  end
end
